﻿using System;
using System.Data;
using System.Runtime.InteropServices;
using Cudafy;
using GASS.CUDA;
using OlaCudafy1.FDTD;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Cudafy.Host;
using OlaCudafy1;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for CSimulationSpaceTest and is intended
    ///to contain all CSimulationSpaceTest Unit Tests
    ///</summary>
	[TestClass()]
	public class CSimulationSpaceTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for TheSources
		///</summary>
		[TestMethod()]
		public void TheSourcesTest()
		{
			try
			{
				GPGPU gpu = CudafyHost.GetDevice(CudafyModes.Target);

				var kindExtent = new KindExtent {X = 2, Y = 3, Z = 5};
				var theSources = new[] {new Source(1, 2, 3, 4F, 5F),};
				var microphones = new[] {new Microphone(6, 7, 8),};
				GridSettings gridSettings = new GridSettings();
				const float dx = 0.01F;
				gridSettings.Setup(dx);
				var target = new CSimulationSpace(kindExtent, gpu, theSources, microphones, gridSettings);

				var expectedSource = new Source(0, 0, 1, 1F, 1F);
				target.TheSources = new[] {expectedSource, };

				var actualSource = target.GetSource(0);
				Assert.AreEqual(expectedSource, actualSource);
				gpu.FreeAll();
			}
			catch (GASS.CUDA.CUDAException e)
			{
				if (e.CUDAError == CUResult.ErrorOutOfMemory)
					throw; // please rewrite test
			} 
			
		}


		/// <summary>
		///A test for CSimulationSpace Constructor
		///</summary>
		[TestMethod()]
		public void CSimulationSpaceConstructorTest_TestPropagationOfPassedArguments()
		{
			var kindExtent = new KindExtent { X = 2, Y = 3, Z = 5 };
			try
			{
				using (var gpu = CudafyHost.GetDevice(CudafyModes.Target))
				{
					try
					{
						ulong m = gpu.FreeMemory;
					}
					catch(CUDAException)
					{
						throw NotImplementedException("Cudafy messed up - but how to fix?");
					}
					var theSource = new Source(1, 2, 3, 4F, 5F);
					var microphone = new Microphone(6, 7, 8);
					GridSettings gridSettings = new GridSettings();
					const float dx = 0.01F;
					gridSettings.Setup(dx);
					var target = new CSimulationSpace(kindExtent, gpu, new[] { theSource }, new[] { microphone }, gridSettings);

					Assert.AreEqual(kindExtent, target.TheSoundField.KindExtent);
					Assert.AreEqual(theSource, target.GetSource(0));
					Assert.AreEqual(microphone, target.GetMicrophone(0));

					// test this too:
					var material = new Wall<byte>(kindExtent, gpu, airValue: 0); // make an identical material

					Assert.AreEqual(material.Length, target.TheWalls.Length); // test only host memory length

					gpu.FreeAll();
				}
			}
			catch (CUDAException e)
			{
				if (e.CUDAError == CUResult.ErrorOutOfMemory)
					throw; // please rewrite test
			}


		}

		private Exception NotImplementedException(string p)
		{
			return new NotImplementedException(p);
		}

		// alt+right is a fine key combination when coding

		/// <summary>
		///A test for CSimulationSpace Constructor
		///</summary>
		[TestMethod()]
		public void CSimulationSpaceConstructorTest_ThrowsExceptionOnInvalidExtent()
		{
			var kindExtent = new KindExtent() {X = 0, Y = 0, Z = 0};
			var gridSettings = new GridSettings();
			const float dx = 0.01F;
			gridSettings.Setup(dx);
			try
			{
				new CSimulationSpace(kindExtent, null, null, null, gridSettings);
				Assert.Fail(
					"CSimulationSpace Constructor did not throw constraint exception even when the extent was 0,0,0!");
			}
			catch (ConstraintException)
			{
			}
		}
	}
}
