﻿using Cudafy;
using OlaCudafy1;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Cudafy.Host;
using OlaCudafy1.FDTD;
using OlaCudafy1.Material;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for DumbDrawTest and is intended
    ///to contain all DumbDrawTest Unit Tests
    ///</summary>
	[TestClass()]
	public class DumbDrawTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for Launch
		///</summary>
		[TestMethod()]
		public void LaunchTest()
		{
			GPGPU gpu;
			try
			{
				gpu = CudafyHost.GetDevice(eGPUType.Cuda);
			}
			catch (GASS.CUDA.CUDAException e)
			{
				if (e.CUDAError == GASS.CUDA.CUResult.ErrorUnknown)
					throw new Exception("WTF...");
				throw;
			}
			Extent extent = new Extent(5, 5, 5);
			var sources = new[] {new Source(-1, -1, -1, 0F, 0F, -1),};
			var microphones = new[] {new Microphone(-1, -1, -1)};
			GridSettings gridSettings = new GridSettings();
			const float dx = 0.01F;
			gridSettings.Setup(dx);
			DumbDraw target = new DumbDraw(
				new CSimulationSpace(extent.KindE, gpu, sources, microphones, gridSettings),
				new DisplaySettings(extent),
				gpu);
			target.Launch();
			var destImg = target.DestinationImage;
			byte expected130 = (byte)(((1 + 3 + 0) / (8F)) * 255F);
			Assert.AreEqual(expected130, destImg[1, 3, 0]);
		}
	}
}
