﻿using System.Collections.Generic;
using System.Xml;
using OlaCudafy1.Material;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Xml.Linq;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for MaterialModelTest and is intended
    ///to contain all MaterialModelTest Unit Tests
    ///</summary>
	[TestClass()]
	public class MaterialModelTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for GuessExtent
		///</summary>
		[TestMethod()]
		[DeploymentItem("OlaCudafy1.exe")]
		public void GuessExtentTest()
		{
			XContainer docRoot = XDocument.Parse(
				"<?xml version=\"1.0\" encoding=\"utf-8\"?>" + Environment.NewLine +
				"<voxel_data>				  " + Environment.NewLine +
				"  <position>				  " + Environment.NewLine +
				"  <object>Sofa</object>	  " + Environment.NewLine +
				"  <material>Cotton</material>" + Environment.NewLine +
				"  <x>0.333333</x>			  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>1.000000</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Window</object>	  " + Environment.NewLine +
				"  <material>Glass</material> " + Environment.NewLine +
				"  <x>0.666666</x>	     	  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>0.666666</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Table</object>	  " + Environment.NewLine +
				"  <material>Wood</material>  " + Environment.NewLine +
				"  <x>1.0000000</x>		      " + Environment.NewLine +
				"  <y>1.3333333</y>		      " + Environment.NewLine +
				"  <z>1.3333333</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" </voxel_data>				  " + Environment.NewLine +
				""
				);
			// x spans from 0.333333 to 1.000000
			// y            1.333333 to 1.666666
			// z            0.666666 to 1.333333
			// and the stepsize is approximately 0.333333
			// therefore extent must be
			//   extent.x 3
			//   extent.y 2
			//   extent.z 5
			Extent_Accessor expected = new Extent_Accessor(3, 2, 3);
			Extent_Accessor actual;
			actual = MaterialModel_Accessor.GuessExtent(docRoot);
			Assert.AreEqual(expected.KindE, actual.KindE);
		}

		/// <summary>
		///A test for GetMaterialNames
		///</summary>
		[TestMethod()]
		[DeploymentItem("OlaCudafy1.exe")]
		public void GetMaterialNamesTest()
		{
			XContainer docRoot = XDocument.Parse(
				"<?xml version=\"1.0\" encoding=\"utf-8\"?>" + Environment.NewLine +
				"<voxel_data>				  " + Environment.NewLine +
				"  <position>				  " + Environment.NewLine +
				"  <object>Sofa</object>	  " + Environment.NewLine +
				"  <material>Cotton</material>" + Environment.NewLine +
				"  <x>0.333333</x>			  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>1.000000</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Window</object>	  " + Environment.NewLine +
				"  <material>Glass</material> " + Environment.NewLine +
				"  <x>0.666666</x>	     	  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>0.666666</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Table</object>	  " + Environment.NewLine +
				"  <material>Wood</material>  " + Environment.NewLine +
				"  <x>1.0000000</x>		      " + Environment.NewLine +
				"  <y>1.3333333</y>		      " + Environment.NewLine +
				"  <z>1.3333333</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Table</object>	  " + Environment.NewLine +
				"  <material>Wood</material>  " + Environment.NewLine +
				"  <x>1.0000000</x>		      " + Environment.NewLine +
				"  <y>1.6666666</y>		      " + Environment.NewLine +
				"  <z>1.3333333</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" </voxel_data>				  " + Environment.NewLine +
				""
				);
			string[] actual = (string[])MaterialModel_Accessor.GetMaterialNames(docRoot);
			Assert.IsTrue(actual.Length == 3);
			var expected = new HashSet<string> {"Cotton", "Glass", "Wood"};
			Assert.IsTrue(expected.SetEquals((new HashSet<string>(actual))));
		}
		
		/// <summary>
		///A test for LoadAndProcessDoc
		///</summary>
		[TestMethod()]
		[DeploymentItem("OlaCudafy1.exe")]
		public void LoadAndProcessDocTest()
		{
			XContainer docRoot = XDocument.Parse(
				"<?xml version=\"1.0\" encoding=\"utf-8\"?>" + Environment.NewLine +
				"<voxel_data>				  " + Environment.NewLine +
				"  <position>				  " + Environment.NewLine +
				"  <object>Sofa</object>	  " + Environment.NewLine +
				"  <material>Cotton</material>" + Environment.NewLine +
				"  <x>0.333333</x>			  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>1.000000</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Window</object>	  " + Environment.NewLine +
				"  <material>Glass</material> " + Environment.NewLine +
				"  <x>0.666666</x>	     	  " + Environment.NewLine +
				"  <y>1.666666</y>	     	  " + Environment.NewLine +
				"  <z>0.666666</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" <position>				  " + Environment.NewLine +
				"  <object>Table</object>	  " + Environment.NewLine +
				"  <material>Wood</material>  " + Environment.NewLine +
				"  <x>1.0000000</x>		      " + Environment.NewLine +
				"  <y>1.3333333</y>		      " + Environment.NewLine +
				"  <z>1.3333333</z>			  " + Environment.NewLine +
				" </position>				  " + Environment.NewLine +
				" </voxel_data>				  " + Environment.NewLine +
				""
				);
			var target = new MaterialModel(docRoot.Document);
			var materials = new Dictionary<string, int>();
			materials["Cotton"] = 0;
			materials["Glass"] = 1;
			materials["Wood"] = 2;
			var expected = new[]
			                  	{
			                  		new Cell(0, 1, 1, materials["Cotton"]),
			                  		new Cell(1, 1, 0, materials["Glass"]),
			                  		new Cell(2, 0, 2, materials["Wood"])
			                  	};
			var actual = target.Cells;
			for (int i = 0; i < actual.Length && i < expected.Length; i++)
			{
				Assert.AreEqual(expected[i].Position, actual[i].Position);
				Assert.AreEqual(expected[i].MaterialIndex, actual[i].MaterialIndex);
			}
		}
		
	}
}
