﻿using Cudafy;
using Cudafy.Host;
using GASS.CUDA;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using OlaCudafy1.FDTD;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for MicrophoneTest and is intended
    ///to contain all MicrophoneTest Unit Tests
    ///</summary>
	[TestClass()]
	public class MicrophoneTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for Microphone Constructor
		///</summary>
		[TestMethod()]
		public void MicrophoneConstructorTest()
		{
			try
			{
				GPGPU gpu = CudafyHost.GetDevice(CudafyModes.Target);

				var microphones = new [] { new Microphone(1, 2, 3), };
				
				Assert.IsNull(microphones[0].DevRecordings);
				Assert.IsFalse(gpu.IsOnGPU(microphones[0].DevRecordings));

				const int len = 40;
				float[] readings = new float[len];
				microphones[0].DevRecordings = gpu.Allocate<float>(len);
				const int expectedDevReadingsLength = 0; // not 40, unfortunately
				Assert.AreEqual(expectedDevReadingsLength, microphones[0].DevRecordings.Length);


				// move the rest of this test to properly named test

				Assert.IsTrue(gpu.IsOnGPU(microphones[0].DevRecordings));

				// before free
				// ===========
				// do not treat or query this pointer except through gpu methods:
					//Assert.IsNotNull(microphones[0].DevReadings); // not null, FORTUNATELY!
				object gpuData = gpu.GetGPUData(microphones[0].DevRecordings);
				Assert.IsNotNull(gpuData,
					"Could not get GPU data, even when it was allocated.");
				Assert.IsTrue(gpu.IsOnGPU(microphones[0].DevRecordings),
					"Data reported to NOT be on GPU, even when it was allocated.");
				try
				{
					gpu.VerifyOnGPU(readings);
					Assert.Fail("Memory on host verified as if it were memory on the GPU.");
					 // should not reach this code
				}
				catch (CudafyHostException)
				{
				}

				// freeing...
				// ===========
				gpu.Free(microphones[0].DevRecordings);

				// after free
				// ===========
				// do not treat or query this pointer except through gpu methods:
					//Assert.IsNull(microphones[0].DevReadings); // not null, unfortunately
				Assert.IsFalse(gpu.IsOnGPU(microphones[0].DevRecordings));
				try
				{
					gpu.GetGPUData(microphones[0].DevRecordings);
					Assert.Fail("Should not be able to get GPU data which has been freed.");
				}
				catch (CudafyHostException e)
				{
					Assert.AreEqual("Data is not on the device.", e.Message);
				}
				try
				{
					gpu.VerifyOnGPU(microphones[0].DevRecordings);
					Assert.Fail(
						"Already *freed* memory (on device) verified as if it were OK!"); // should not reach this code
				}
				catch (CudafyHostException)
				{
				}

				gpu.FreeAll();
			}
			catch (GASS.CUDA.CUDAException e)
			{
				if (e.CUDAError == CUResult.ErrorOutOfMemory)
					throw; // please rewrite test
			}
		}
	}
}
