﻿using OlaCudafy1.FDTD;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Cudafy.Host;
using Cudafy;
using OlaCudafy1.Material;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for SimpleCalcTest and is intended
    ///to contain all SimpleCalcTest Unit Tests
    ///</summary>
	[TestClass()]
	public class SimpleCalcTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		/// Another test for ManualLaunch; simpler
		///</summary>
		[TestMethod()]
		public void ManualLaunchTest()
		{
			const float epsilon = 0.00001F; // allowable error for this test

			var extent = new Extent(3, 3, 2);
			KindExtent ke = extent.KindE;
			GPGPU gpu = CudafyHost.GetDevice(eGPUType.Cuda);
			
			try {
				var theGridSettings = new GridSettings();
				const float dX = .01F;
				theGridSettings.Setup(dX);

				var pHost = new float[extent.X,extent.Y,extent.Z];
				const float a = 10;
				const float b = 0; // -10;
				const float c = 0; //-20;
				const float d = 0; //20;
				//x  y  z
				pHost[1, 1, 1] = a;
				pHost[2, 1, 1] = b;
				pHost[1, 2, 1] = c;
				pHost[2, 2, 1] = d;
				float[,,] devP = gpu.Allocate(pHost);
				Assert.IsTrue((gpu.IsOnGPU(devP)));
				gpu.CopyToDevice(pHost, devP);
				Assert.IsTrue((gpu.IsOnGPU(devP)));

				var horiz = new float[extent.X,extent.Y,extent.Z];
				float[,,] devHoriz = gpu.Allocate(horiz);
				var vert = new float[extent.X,extent.Y,extent.Z];
				float[,,] devVert = gpu.Allocate(vert);
				var zSpeeds = new float[extent.X,extent.Y,extent.Z];
				float[,,] devZSpeeds = gpu.Allocate(zSpeeds);

				#region this works

				var pCopyBack = new float[extent.X,extent.Y,extent.Z];
				gpu.CopyFromDevice(devP, pCopyBack);
				Assert.AreEqual(pHost[1, 1, 1], pCopyBack[1, 1, 1]);
				Assert.AreEqual(a, pCopyBack[1, 1, 1]);

				#endregion

				// dummy sources array
				Source[] sources = new Source[] {new Source(),};
				Source[] devSources = gpu.Allocate(sources);
				gpu.CopyToDevice(sources, devSources);
				MicrophoneStruct[] mics = new MicrophoneStruct[]{new MicrophoneStruct(),} ;
				MicrophoneStruct[] devMics = gpu.Allocate(mics);
				gpu.CopyToDevice(mics, devMics);

				var recordings = new float[mics.Length,1];
				//for (int index = 0; index < recordings.Length; index++)
					//recordings[index] = new float[1]; // 1 iteration
				float[,] devRecordings = gpu.Allocate(recordings);

				GridSettings[] gridAr = new[] {theGridSettings};
				GridSettings[] devTheGridSettingsArray = gpu.Allocate(gridAr);
				gpu.CopyToDevice(gridAr, devTheGridSettingsArray);
				Assert.IsTrue(gpu.IsOnGPU(devTheGridSettingsArray));

				var materialData = new byte[extent.X, extent.Y, extent.Z];
				var adjacentSurfaces = new byte[extent.X, extent.Y, extent.Z];
				byte[, ,] devMaterialData = gpu.Allocate(materialData);
				byte[, ,] devAdjacentSurfaces = gpu.Allocate(adjacentSurfaces);
				var absorption = new float[sizeof (byte)];
				float[] devAbsorption = gpu.Allocate(absorption);
				var target = new SimpleCalc(null, gpu);
				target.ManualLaunch(
					new dim3(1, 1),
					new dim3(16, 16),
					devP,
					devHoriz,
					devVert,
					devZSpeeds,
					ke,
					devTheGridSettingsArray,
					devSources,
					devMics,
					devRecordings,
					devMaterialData,
					devAdjacentSurfaces,
					devAbsorption,
					0
					);

				// copy back result
				var pAfterDeviceCalc = new float[extent.X,extent.Y,extent.Z];
				gpu.CopyFromDevice(devP, pAfterDeviceCalc);

				// calculate expected result, store in p (will mutate p)
				var dim = new dim3(extent.X, extent.Y);
				GGrid grid = new GGrid(dim);
				var block = new GBlock(grid, grid.Dim, 0, 0);
				for (byte part = 1; part <= 2; part++)
				{
					for (int y = 0; y < dim.y; y++)
					{
						for (int x = 0; x < dim.x; x++)
						{
							//if (part == 1 && y == 1 && x == 2)
							//lol = 1;
							GThread thread = new GThread(x, y, block);
							SimpleCalc.SimpleCalcKernel(thread, pHost, horiz, vert,
							                            zSpeeds, ke, gridAr, part, sources,
														mics, recordings,
														materialData,
														adjacentSurfaces,
														absorption,
														0);
						}
					}
				}
				var expectedP111 = pHost[1, 1, 1];
				//- (a - b) * theGridSettings.CU * theGridSettings.CP
				//- (a - c) * theGridSettings.CU * theGridSettings.CP
				//- (a) * theGridSettings.CU * theGridSettings.CP
				//+ (0 - a) * theGridSettings.CU * theGridSettings.CP
				//+ (0 - a) * theGridSettings.CU * theGridSettings.CP
				//+ (0 - a) * theGridSettings.CU * theGridSettings.CP;

				Assert.AreEqual(pHost.Length, pAfterDeviceCalc.Length);
				for (int z = 0; z < ke.Z; z++)
				{
					for (int y = 0; y < ke.Y; y++)
					{
						for (int x = 0; x < ke.X; x++)
						{
							Assert.AreEqual(pHost[x, y, z], pAfterDeviceCalc[x, y, z], epsilon);
						}
					}
				}
				Assert.AreEqual(expectedP111, pAfterDeviceCalc[1, 1, 1], epsilon);
			}
			finally
			{
				gpu.FreeAll();

			}
		}
	}
}
