﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using OlaCudafy1.FDTD;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for ToolboxTest and is intended
    ///to contain all ToolboxTest Unit Tests
    ///</summary>
	[TestClass()]
	public class ToolboxTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for dBrefSPL0ToPressure in the 60 dB case
		///</summary>
		[TestMethod()]
		public void dBrefSPL0ToPressureTest_60dB()
		{
			float dB = 60F;
			float expected = 20f*0.001f; // 20 mPa ~ 60 dB
			float actual;
			actual = Toolbox.DecibelsRefSPL0ToPressure(dB);
			Assert.AreEqual(expected, actual, delta: 0.0000001);
		}

		/// <summary>
		///A test for dBrefSPL0ToPressure in the 0 dB case
		///</summary>
		[TestMethod()]
		public void dBrefSPL0ToPressureTest_0dB()
		{
			float dB = 0F;
			float expected = AirIntrinsicProperties.SPL0;
			float actual;
			actual = Toolbox.DecibelsRefSPL0ToPressure(dB);
			Assert.AreEqual(expected, actual, delta: 0.000000000001);
		}

		/// <summary>
		///A combined test for PressureTodBrefSPL0 and dBrefSPL0ToPressure
		///</summary>
		[TestMethod()]
		public void PressureTodBrefSPL0Test()
		{
			const float pressureAmplitude = 14.35f;
			const float expected = pressureAmplitude;
			float dB = Toolbox.PressureTodBrefSPL0(pressureAmplitude);
			float actual = Toolbox.DecibelsRefSPL0ToPressure(dB);
			Assert.AreEqual(expected, actual, delta:0.002);
		}
	}
}
