﻿using OlaCudafy1;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;

namespace OlaCudafy1.Test
{
    
    
    /// <summary>
    ///This is a test class for WordWrapTextTest and is intended
    ///to contain all WordWrapTextTest Unit Tests
    ///</summary>
	[TestClass()]
	public class WordWrapTextTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for WrapTextDefault: Typical case
		///</summary>
		[TestMethod()]
		public void WrapTextDefaultTest()
		{
			const string text = "Fiberglass boards and blankets, 5.1 cm fiberglass, panels with plastic sheet wrapping and perforated metal facing, as installed Ref. Harris:Handbook of Acoustical Measurements and Noise Control, McGraw Hill 1991";
			var expected = new List<string>
			                        	{
			                        		"Fiberglass boards and blankets,",
			                        		"5.1 cm fiberglass, panels with",
			                        		"plastic sheet wrapping and",
			                        		"perforated metal facing, as",
			                        		"installed Ref. Harris:Handbook",
			                        		"of Acoustical Measurements",
			                        		"and Noise Control, McGraw",
			                        		"Hill 1991"
			                        	};
			var actual = (List<string>) WordWrapText.WrapTextDefault(text);
			Assert.AreEqual(expected.Count, actual.Count);
			for (int i = 0; i < expected.Count; i++)
				Assert.AreEqual(expected[i], actual[i]);
		}


		/// <summary>
		///A test for WrapTextDefault: Funny case
		///</summary>
		[TestMethod()]
		public void WrapTextDefaultTestFunnyCase()
		{
			const string text = "Audience on wooden chairs, 2 per sq. m. (Ref. 20)";
			var expected = new List<string>
			                        	{
			                        		"Audience on wooden chairs,",
			                        		"2 per sq. m. (Ref. 20)"
			                        	};
			var actual = (List<string>)WordWrapText.WrapTextDefault(text);
			Assert.AreEqual(expected.Count, actual.Count);
			for (int i = 0; i < expected.Count; i++)
				Assert.AreEqual(expected[i], actual[i]);
		}


		/// <summary>
		///A test for WrapTextDefault: Border case: empty string
		///</summary>
		[TestMethod()]
		public void WrapTextDefaultTestBorderEmptyString()
		{
			const string text = "";
			var expected = new List<string> {""};
			var actual = (List<string>)WordWrapText.WrapTextDefault(text);
			Assert.AreEqual(expected.Count, actual.Count);
			for (int i = 0; i < expected.Count; i++)
				Assert.AreEqual(expected[i], actual[i]);
		}


		/// <summary>
		///A test for WrapTextDefault: Border case: No spaces
		///</summary>
		[TestMethod()]
		public void WrapTextDefaultTestBorderNoSpaces()
		{
			const string text = "detteeren-kjempelang-streng-uten-noen-mellomrom-s[detsa!";
			var expected = new List<string> { text };
			var actual = (List<string>)WordWrapText.WrapTextDefault(text);
			Assert.AreEqual(1, actual.Count);
			Assert.AreEqual(expected[0], actual[0]);
		}
	}
}
