﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace OlaCudafy1
{
	/// <summary>
	/// Absorber coefficients for the  8 frequency bands
	/// and a ToString method
	/// </summary>
	class AbsorberCoefficients
	{
		private readonly float[] _coefficients;

		/// <summary>
		/// Creates a new instance of the AbsorberCoefficient class
		/// with the supplied argument set as coefficients.
		/// </summary>
		/// <param name="coefficients"></param>
		public AbsorberCoefficients(float[] coefficients)
		{
			_coefficients = coefficients;
		}

		/// <summary>
		/// A simple human-readable string of the absorption coefficients.
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return _coefficients.Aggregate("", (current, coefficient) =>
						current + (coefficient.ToString("0.00") + " "))
						.TrimEnd(new[] { ' ' });
		}

		/// <summary>
		/// Gets absorption coefficient at 'index'.
		/// If index is outside array dimensions, 0 is returned.
		/// </summary>
		/// <param name="index"></param>
		/// <returns></returns>
		public float this[int index]
		{
			get
			{
				if (0 <= index && index < _coefficients.Length)
					return _coefficients[index];
				return 0f;
			}
		}

		/// <summary>
		/// Returns a new BandCoefficient with abs.coeff. data from this instance.
		/// </summary>
		/// <returns></returns>
		public BandCoefficients ToBandCoefficients()
		{
			return new BandCoefficients(_coefficients);
		}
	}
}
