﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace OlaCudafy1
{
	internal class AbsorberDatabaseEntry
	{
		// acquired from the original absorption database
		internal int ID { get; set; }
		internal string Description { get; set; }
		internal AbsorberCoefficients AbsorberCoefficients { get; set; }

		// derived data
		internal RindelImpedanceEstimation.AbsorberType AbsorberType;
		internal float R;
		internal float m;
		internal float d;

		public AbsorberDatabaseEntry(int number, string description, float[] coefficients)
		{
			ID = number;
			Description = description;
			AbsorberCoefficients = new AbsorberCoefficients(coefficients);

			AbsorberType = RindelImpedanceEstimation.GuessAbsorberType(coefficients);
			var rindelImpedanceEstimation =
				new RindelImpedanceEstimation(coefficients, 413.3f); // TODO remove magic number
			R = rindelImpedanceEstimation.EstimatedResistanceTerm;
			try
			{
				m = rindelImpedanceEstimation.EstimateM();
				d = rindelImpedanceEstimation.EstimateD(340f); // TODO remove magic number
			}
			catch (NotImplementedException)
			{
				m = Single.NaN;
				d = Single.NaN;
			}
		}

		internal object[] ToArray()
		{
			return new object[] {ID, AbsorberCoefficients.ToString(),
			                     AbsorberTypeString, ImpedanceModelString, Description,
			                     this};
		}

		protected string AbsorberTypeString
		{
			get { return AbsorberType.ToString(); }
		}

		internal static object[][] NamesTypesArray()
		{
			return new[] {new object[] {"ID", typeof(int)},
			              new object[] {"Coefficients", typeof(string)},
			              new object[] {"Type", typeof(string)},
			              new object[] {"Impedance model", typeof(string)},
			              new object[] {"Description", typeof(string)},
			              new object[] {"entry", typeof(AbsorberDatabaseEntry)},
			             };
		}

		public object ImpedanceModelString
		{
			get
			{
				return String.Format("R: {0:0.0}, m: {1:0.00}, d: {2:0.000}",
									 R, m, d);
			}
		}
	}
}
