﻿/* This file defines the properties of air
 * for use in the numerical simulation
 */

namespace OlaCudafy1.FDTD
{
	/// <summary>
	/// Acoustic parameters, readonlyants
	/// </summary>
	struct AirIntrinsicProperties
	{
		/// <summary>
		/// Density of air, rho, in kg/m3, at 20 degrees celcius
		/// </summary>
		internal static readonly float R0 = 1.2041f;

		/// <summary>
		/// Approximate speed of sound [m/s] in air at 20 degrees celcius
		/// </summary>
		internal static readonly float C0 = 343.0f;

		/// <summary>
		/// Acoustic impedance of air at 20 degrees celcius
		/// </summary>
		internal static readonly float Impedance = R0*C0;

		/// <summary>
		/// Volume elasticity / bulk modulue E of air [Pa] = [N/m2]
		/// </summary>
		internal static readonly float Rk = R0*C0*C0;

		/// <summary>
		/// Sound pressure level of 0dB ref regular reference for regular acoustics, which is 20 microPascals.
		/// </summary>
		internal const float SPL0 = (float)((20.0 * 0.001 * 0.001) * 1.0f);

		/// <summary>
		/// Sound pressure leevl of 60dB ref SPL0
		/// </summary>
		internal const float dB60 = 0.020f;
	}
}