﻿using Cudafy;
using OlaCudafy1.Material;

namespace OlaCudafy1.FDTD
{
	internal enum SliceDimension
	{
		X = 0,
		Y = 1,
		Z = 2,
		Undefined
	}

	internal class DisplaySettings
	{
		#region Delegates

		public delegate void ChangedEventHandler();

		#endregion

		private readonly Extent _extent;

		private int _displaySlice;
		private int _sliceDimensionInt;
		private float _weightBothInDecibels;
		private float _weightPressureInDecibels;

		public DisplaySettings(Extent extent)
		{
			_extent = extent;
			_displaySlice = 0;
			WeightPressureInDecibels = 0F;
			WeightBothInDecibels = 0F;
		}

		public float WeightPressureInDecibels
		{
			get { return _weightPressureInDecibels; }
			set
			{
				_weightPressureInDecibels = value;
				OnChanged();
			}
		}


		// ReSharper disable InconsistentNaming

		// ReSharper restore InconsistentNaming

		public int NumberOfSlices
		{
			get { return _extent[SliceDimension]; }
		}

		public int DisplaySlice
		{
			get { return _displaySlice; }
			set
			{
				SafeSetDisplaySlice(value);
				OnChanged();
			}
		}

		public float WeightBothInDecibels
		{
			get { return _weightBothInDecibels; }
			set
			{
				_weightBothInDecibels = value;
				OnChanged();
			}
		}


		public SliceDimension SliceDimension
		{
			get
			{
				return (SliceDimension) _sliceDimensionInt;
				//switch (_sliceDimensionInt)
				//{
				//    case 0:
				//        return SliceDimension.X;
				//    case 1:
				//        return SliceDimension.Y;
				//    case 2:
				//        return SliceDimension.Z;
				//    default:
				//        return SliceDimension.Undefined;
				//}
			}
			set
			{
				_sliceDimensionInt = (int) value;
				//_sliceDimensionInt = (int)Enum.ToObject(typeof(SliceDimension), value);
				//_sliceDimensionInt = value;
				SafeSetDisplaySlice(DisplaySlice);
				OnChanged();
			}
		}

		public float ScalePressure
		{
			get { return Toolbox.DecibelsRef1ToPressure(_weightPressureInDecibels); }
		}

		public float ScaleBoth
		{
			get { return Toolbox.DecibelsRef1ToPressure(_weightBothInDecibels); }
		}

		public int SliceDimensionInt
		{
			get { return _sliceDimensionInt; }
		}

		private void SafeSetDisplaySlice(int slice)
		{
			if (slice > NumberOfSlices - 1)
				_displaySlice = NumberOfSlices - 1;
			_displaySlice = slice;
		}

		public event ChangedEventHandler Changed;

		protected virtual void OnChanged()
		{
			if (Changed != null)
				Changed();
		}
	}
}