﻿using Cudafy;

namespace OlaCudafy1.FDTD
{
	// see further down for the FDTDGridSpec class

	//[Cudafy]
	struct FDTDGridSpecStruct
	{
		internal int Rank;

		internal int[] Dimensions;

		public int GetRank()
		{
			return Rank;
		}

		public int GetX()
		{
			return Rank > 0 ? Dimensions[0] : 1;
		}

		//[CudafyIgnore]
		public int X
		{
			get
			{
				return Rank > 0 ? Dimensions[0] : 1;
			}
		}

		//[CudafyIgnore]
		public int Y
		{
			get
			{
				return GetY();
			}
		}

		public int GetY()
		{
			return Rank > 1 ? Dimensions[1] : 1;
		}

		public int GetZ()
		{
			return Rank > 3 ? Dimensions[3] : 1;
		}

		//[CudafyIgnore]
		public int Z
		{
			get
			{
				return GetZ();
			}
		}

	}

	class FDTDGridSpec
	{
		public FDTDGridSpecStruct Struct
		{
			get
			{
				return _internalStruct;
			}
		}

		FDTDGridSpecStruct _internalStruct;

		public FDTDGridSpec(int rank)
		{
			_internalStruct = new FDTDGridSpecStruct();
			Rank = rank;
			Dimensions = new int[Rank];
		}

		internal int[] Dimensions
		{
			get
			{
				return _internalStruct.Dimensions;
			}
			set
			{
				_internalStruct.Dimensions = value;
			}
		}

		public FDTDGridSpec(int rank, int[] dimensions)
			: this(rank)
		{
			Dimensions = dimensions;
		}

		public int Rank
		{
			get { return Struct.Rank; }
			private set { _internalStruct.Rank = value; }
		}


		public int X
		{
			get { return Dimensions[0]; }
			set { _internalStruct.Dimensions[0] = value; }
		}

		public int Y
		{
			get { return Dimensions[1]; }
			set { _internalStruct.Dimensions[1] = value; }
		}

		public int Z
		{
			get { return Dimensions[2]; }
			set { _internalStruct.Dimensions[2] = value; }
		}

		internal dim3 ToDim3()
		{
			var dim = new dim3(X, Y);
			// TODO how to set Z?
			return dim;
		}
	}
}
