﻿using System;
using Cudafy;
using OlaCudafy1.FDTD;

namespace OlaCudafy1.FDTD
{
	[Cudafy]
	public struct GridSettings
	{
		internal float Boom;
		internal float CP; // todo document 
		internal float CU; // todo document
		internal float DT;
		internal float DX; // =DY=DZ -- in this program, the FDTD grid is completely square/cubic
		internal float FS;

		// dependants:
		internal float GridMaxFreq; // max frequency practically representable in the grid
		internal float GridNyquistFreq; // max frequency theoretically representable in the grid
		internal int MinCellsPerWavelength;

		// dependands for Mur boundary condition:
		internal float MurCoefficient1;
		internal float MurCoefficient2;
		internal float MurCoefficient3;
		private float _rmax;

		// copies
		internal float R0;
		internal float Rk;
		internal float C0;

		internal float SoundPressure;

		// TODO remove:
		internal float Impedance;

		[CudafyIgnore]
		private static float CalculateTimeSamplingFrequencyWithMargin(float dx)
		{
			float dy = dx;
			float dz = dx;
			float dmax = Math.Max(Math.Max(dx, dy), dz);
			return (float) (AirIntrinsicProperties.C0*2.0/dmax);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="dx">grid size in meters: x, y and z directions (typically 0.01)</param>
		[CudafyIgnore]
		internal void Setup(
			float dx
			)
		{
			Impedance = 1.2041F*343;

			SoundPressure = C0*R0;//????

			R0 = AirIntrinsicProperties.R0;
			Rk = AirIntrinsicProperties.Rk;
			C0 = AirIntrinsicProperties.C0;

			DX = dx;
			FS = CalculateTimeSamplingFrequencyWithMargin(DX);

			// dependants:
			DT = 1.0F/FS;
			CU = DT/DX/R0;
			CP = Rk*DT/DX;

			// the number 6, number of grid cells per wave
			// can be 5, but should be between 10 and 20, 30, 40 even 60 or more
			// for smooth simulations
			MinCellsPerWavelength = 6;

			GridMaxFreq = AirIntrinsicProperties.C0/MinCellsPerWavelength/DX;
			// absolute upper frequency limit representable in the grid
			// (actually first non-representable frequency)
			GridNyquistFreq = AirIntrinsicProperties.C0/2/DX;

			// Definitions for MUR boundary condition
			// for PML/Mur from fdm_3d_pml.f90
			_rmax = (float) 14000.0; // what's this?
			MurCoefficient1 = ((C0*DT + DX - _rmax*DX*DT)/(C0*DT + DX + _rmax*DX*DT));
			MurCoefficient2 = ((C0*DT - DX + _rmax*DX*DT)/(C0*DT + DX + _rmax*DX*DT));
			MurCoefficient3 = ((C0*DT - DX - _rmax*DX*DT)/(C0*DT + DX + _rmax*DX*DT));

			Boom = DX/DT/C0; // sonic boom
		}
	}
}