﻿using Cudafy;
using Cudafy.Host;
using Cudafy.Translator;

namespace OlaCudafy1.FDTD
{
	public class RippleGpu
	{
		public RippleGpu()
		{
		}

		public const int DIM = 1024;

		private byte[] _devBitmap;

		private GPGPU _gpu;

		private dim3 _blocks;

		private dim3 _threads;

		public void Initialize(int bytes)
		{
			CudafyModule km = CudafyTranslator.Cudafy();

			_gpu = CudafyHost.GetDevice(CudafyModes.Target);
			_gpu.LoadModule(km);

			_devBitmap = _gpu.Allocate<byte>(bytes);

			_blocks = new dim3(DIM / 16, DIM / 16);
			_threads = new dim3(16, 16);
		}

		public void Execute(byte[] resultBuffer, int ticks)
		{
			_gpu.Launch(_blocks, _threads).Kernel(_devBitmap, ticks);
			_gpu.CopyFromDevice(devArray: _devBitmap, hostArray: resultBuffer);
		}

		[Cudafy]
		public static void Kernel(GThread thread, byte[] ptr, int ticks)
		{
			// map from threadIdx/BlockIdx to pixel position
			int x = thread.threadIdx.x + thread.blockIdx.x * thread.blockDim.x;
			int y = thread.threadIdx.y + thread.blockIdx.y * thread.blockDim.y;
			int offset = x + y * thread.blockDim.x * thread.gridDim.x;

			// now calculate the value at that position
			float dx = x - DIM / 2;
			float dy = y - DIM / 2;
			float distanceFromCenter = GMath.Sqrt(dx * dx + dy * dy);

			// calculate gray value for that pixel (colors not implemented)
			byte grey = (byte)(128.0f + 127.0f * GMath.Cos(distanceFromCenter / 10.0f - ticks / 7.0f) /
												 (distanceFromCenter / 10.0f + 1.0f));

			ptr[offset * 4 + 0] = grey; // R
			ptr[offset * 4 + 1] = grey; // G
			ptr[offset * 4 + 2] = grey; // B
			ptr[offset * 4 + 3] = 255; // alpha
		}

		public void ShutDown()
		{
			_gpu.FreeAll();
		}
	}
}
