﻿using Cudafy;
using OlaCudafy1.FDTD;

namespace OlaCudafy1.FDTD
{
	[Cudafy]
	public struct Source
	{
		public readonly int X, Y, Z;
		public readonly float Frequency;
		public readonly float PressureAmplitude;
		public readonly int InvokeAtGeneration;
		public readonly int Type;
		
		// TODO test that the struct init default values actually makes a disabled source
		///// <summary>
		///// Default constructor. Makes a source which is disabled.
		///// </summary>
		//Source()
		//{
		//    X = -1;
		//    Y = -1;
		//    Z = -1;
		//    Frequency = 0F;
		//    PressureAmplitude = 0F;
		//    InvokeAtGeneration = -1;
		//}

		public Source(
			int x,
			int y,
			int z,
			float frequency,
			float pressureAmplitude = AirIntrinsicProperties.dB60,
			int invokeAtGeneration = 0,
			int type = 0)
		{
			X = x;
			Y = y;
			Z = z;
			Frequency = frequency;
			PressureAmplitude = pressureAmplitude;
			InvokeAtGeneration = invokeAtGeneration;
			Type = type;
		}

		/// <summary>
		/// For mutating a source. Specify arguments to change, but prefix them by by name:
		/// </summary>
		[CudafyIgnore]
		public void Mutate(
			int x = -1,
			int y = -1,
			int z = -1,
			float frequency = float.NaN,
			float pressureAmplitude = float.NaN,
			int type = -1)
		{
			Source source = this;
			int newX = x == -1 ? source.X : x;
			int newY = y == -1 ? source.Y : y;
			int newZ = z == -1 ? source.Z : z;
			float newFrequency = float.IsNaN(frequency) ? source.Frequency : frequency;
			float newPressureAmplitude = float.IsNaN(pressureAmplitude) ? source.PressureAmplitude : pressureAmplitude;
			int newType = type == -1 ? source.Type : type;
			var newSource = new Source(
				newX,
				newY,
				newZ,
				newFrequency,
				newPressureAmplitude,
				newType
				);
			this = newSource;
		}
	}

}
