﻿using System.Collections.Generic;
using System.Linq;
using Cudafy;
using Cudafy.Translator;

namespace OlaCudafy1.FDTD
{
	/// <summary>
	/// Better than CudafyTranslator.Cudafy
	/// </summary>
	class SuperCudafier
	{
		/// <summary>
		/// Cudafies the types in the provided list, *in order*
		/// </summary>
		/// <param name="types"></param>
		/// <param name="replaceList">Replacements to be done in the source code (be careful!!!)</param>
		/// <returns></returns>
		public static CudafyModule Cudafy(System.Type[] types, Dictionary<string, string> replaceList=null)
		{
			//return CudafyTranslator.Cudafy(types);

			System.Diagnostics.StackTrace st = new System.Diagnostics.StackTrace(1);
			System.Diagnostics.StackFrame sf = st.GetFrame(0);
			string nameOfCaller = "complete";
			nameOfCaller = string.Format("{0}.{1}.cdfy",
				sf.GetMethod().DeclaringType.FullName,
				sf.GetMethod().Name);

			
			CudafyModule complete = CudafyModule.TryDeserialize(nameOfCaller);
			bool alwaysRecompileKernelModule = false;
#if DEBUG
			//alwaysRecompileKernelModule = true;
#endif
			if (alwaysRecompileKernelModule
				|| complete == null
				|| !complete.TryVerifyChecksums())
			{
				complete = new CudafyModule();
				CudafyModule module = new CudafyModule();
				foreach (var type in types)
				{
					#region help for understanding errors
					// Errors reported from (seems to originate) here (*) and their solution (->):
					// *NullReferenceException: "Object reference not set to an instance of an object."
					//   -> Constant device memory must be marked as 'public', not 'internal' or 'private', eg.
					//      public static Item[] DevPile = new Item[NumberOfItems];
					// *CudafyLanguageException: "Array create expressions are not supported."
					//   -> Struct to be cudafied cannot contain and instantiate regular array
					// *CudafyLanguageException: "Constant memory variables must be initialized in-line."
					//   -> 
					// *ArgumentNullException: "Value cannot be null. Parameter name: source"
					//   -> must not mark struct with [Cudafy] which has array that must be marked with [Cudafy], i.e. /do/
					//        public struct Container
					//      instead of
					//        [Cudafy] public struct Container
					//      when the struct contains
					//        [Cudafy] public static Item[] DevItems = new Item[Count];
					//   -> must not mark array with [Cudafy] is inside struct that must be marked with [Cudafy], i.e. /do/
					//        public static Item[] DevItems = new Item[Count];
					//      instead of
					//        [Cudafy] public static Item[] DevItems = new Item[Count];
					//      when the struct is marked as
					//        [Cudafy] public struct Container
					// *TargetException: "Non-static field requires a target."
					//   -> Did you forget to Cudafy or [Cudafy]-mark a struct which you send to the kernel,
					//      or any of the struct's subparts?
					//   -> Or does the kernel have another signature than what you use when calling it dynamically?
					//   -> Also, mark constant device memory variables with static also
					// ---
					// Use (mouseover) 'name' to find the name of the type which contains the error! :)
					#endregion
					string name = type.Name;
					module =
						CudafyTranslator.Cudafy(ePlatform.Auto, eArchitecture.sm_12,
												cudaVersion: null, compile: false, types: type);
					// do NOT compile
					complete.CudaSourceCode += module.CudaSourceCode;
				}
				// move over all else to complete -- highly crucial
				module.CudaSourceCode = complete.CudaSourceCode;
				if (replaceList != null)
					module.CudaSourceCode = Replace(module.CudaSourceCode, replaceList);
				complete = module;

				#region help for understanding errors 2
				// Errors reported from (seems to originate) here (*) and their solution (->):
				// *?: "the size of an array must be greater than zero"
				//   -> did this error appear upon second launching of the kernel?
				// *CudafyCompileException: "error: no operator "[]" matches these operands. operand types are: Item [ int ]"
				//   -> GPU constant memory definitions must be instantiated, i.e. like this:
				//      [Cudafy] public static Item[] DevPile = new Item[NumberOfItems];
				//      and not like this:
				//      [Cudafy] public static Item[] DevPile;
				// *CudafyCompileException: "error: attribute "__global__" does not apply here"
				//   -> Did you forget to Cudafy or [Cudafy]-mark a struct which you send to the kernel,
				//      or any of the struct's subparts?
				// *CudafyCompileException: "error: data member initializer is not allowed"
				//   -> 
				// *CudafyCompileException: "error: linkage specification is not allowed"
				//   -> Functions allocating on device are to be [CudafyIgnore]d or at least not [Cudafy]d.
				//      Also, it more eagerly generates __global__ function definitions when your functions
				//      return void, so make them return int instead
				// *CudafyCompileException: "error: identifier "Inner" is undefined"
				//   -> Forgot [Cudafy] attribute on the struct Inner?
				// *CudafyCompileException: "error: identifier "DevInnerLen0" is undefined"
				//   -> Cudafy does not seem to handle lengths of inner things...???
				//      Make your own length variable. Nah, nvm that.
				#endregion
				string compileCommandLine =
					complete.Compile(eGPUCompiler.CudaNvcc,
					deleteGeneratedCode: false);
				complete.Serialize(nameOfCaller); // we serialize after compiling
			}

			return complete;
		}

		private static string Replace(string sourceCode, Dictionary<string, string> replaceList)
		{
			return replaceList.Aggregate(sourceCode, (current, pair) => current.Replace(pair.Key, pair.Value));
		}
	}
}
