﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;

namespace OlaCudafy1
{
	public partial class FormAbsorberDatabase : Form
	{
		private static OpenFileDialog _openAbsorptionTextFileDialog;
		private AbsorberDatabase _absorberDatabase;
		private DataView _myView;
		private DataTable _table;
		private bool _considerSend;

		public FormAbsorberDatabase()
		{
			InitializeComponent();
		}

		/// <summary>
		/// (To be called when the user has chosen a file to load in
		/// the file load dialog.)
		/// Collects the filename specified by the user and
		/// and proceeds by loading that file.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OpenAbsorptionTextFileDialogFileOk(object sender, CancelEventArgs e)
		{
			if (_openAbsorptionTextFileDialog == null)
				return;
			string filename = _openAbsorptionTextFileDialog.FileName;
			LoadAbsorptionDatabaseAndPrepareDatagridview(filename);
		}

		/// <summary>
		/// Loads an absorption database from the given filename and
		/// connects it to the data grid view, which it also prepares
		/// </summary>
		/// <param name="filename"></param>
		private void LoadAbsorptionDatabaseAndPrepareDatagridview(string filename)
		{
			// try to load absorption database from .li8-file,
			// which is the format in which Multiconsult stores their
			// absorption database
			_absorberDatabase = new AbsorberDatabase(filename);

			// fill the data grid view with the absorption database
			if (_absorberDatabase.Database == null ||
			    _absorberDatabase.Database.Count <= 0)
				return;

			// prepare a datatable for insertion of the material absorption data
			_table = new DataTable();

			// prepare the table's columns' names and datatypes
			foreach (var nameType in AbsorberDatabaseEntry.NamesTypesArray())
			{
				_table.Columns.Add((string) nameType[0], (Type) nameType[1]);
			}

			// add the data table to a data set (this might not be needed)
			var dataSet = new DataSet();
			dataSet.Tables.Add(_table);

			// iterate through the absorber database and insert each entry into 
			// the newly prepared data table
			foreach (AbsorberDatabaseEntry databaseMaterialEntry in _absorberDatabase.Database)
			{
				_table.Rows.Add(_table.NewRow().ItemArray = databaseMaterialEntry.ToArray());
			}
			_table.AcceptChanges();

			dataGridView1.CellToolTipTextNeeded += DataGridView1OnCellToolTipTextNeeded;

			// show the data in the data grid view
			_myView = _table.DefaultView;
			//new DataView(,)
			dataGridView1.DataSource = _myView;

			// todo. this is not really how to do it:
			DataGridViewColumn hideThisColumn = dataGridView1.Columns["entry"];
			if (hideThisColumn != null) hideThisColumn.Visible = false;

			dataGridView1.DefaultCellStyle.WrapMode = DataGridViewTriState.True;

			DataGridViewColumn gridViewColumn = dataGridView1.Columns["Description"];
			if (gridViewColumn != null)
				gridViewColumn.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
		}

		private void DataGridView1OnCellToolTipTextNeeded(object sender,
		                                                  DataGridViewCellToolTipTextNeededEventArgs e)
		{
			if (!(0 <= e.RowIndex && e.RowIndex < dataGridView1.RowCount)) return;
			DataGridViewRow row = dataGridView1.Rows[e.RowIndex];
			if (4 >= row.Cells.Count) return;
			string toolTip = row.Cells[4].Value.ToString();
			IEnumerable<string> toolTipLines = WordWrapText.WrapTextDefault(toolTip);
			e.ToolTipText = toolTipLines.Aggregate((aggregate, line) => aggregate + Environment.NewLine + line);
		}

		/// <summary>
		/// Refilters the contents of the data grid view
		/// to reflect the search keywords
		/// </summary>
		private void TextBoxSearchKeyUp(object sender, KeyEventArgs e)
		{
			if (_myView == null) return;

			string outputInfo = string.Empty;
			string[] keyWords = textBoxSearch.Text.Split(' ');

			foreach (string word in keyWords)
			{
				if (word.IndexOfAny(new[] {'\'', '"'}) > -1)
					continue;
				if (outputInfo.Length != 0)
					outputInfo += " AND ";
				outputInfo += "(";

				outputInfo += string.Format("Description LIKE '%{0}%'", word);

				int wordAsInt;
				if (int.TryParse(word, out wordAsInt))
					outputInfo += string.Format(" OR ID = '{0}'", wordAsInt);

				outputInfo += ")";
			}

			//Applies the filter to the DataView
			_myView.RowFilter = outputInfo;

			labelResultCount.Text = _myView.Count.ToString();
		}


		/// <summary>
		/// Autoresizes the rows of the data grid view to make
		/// all cells in the view fit
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void buttonFixRows_Click(object sender, EventArgs e)
		{
			dataGridView1.AutoResizeRows(DataGridViewAutoSizeRowsMode.AllCells);
		}

		internal event AbsorbentSelectedDelegate AbsorbentDoubleClickedEvent;

		private void dataGridView1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
		{
			DataGridViewCell cell = dataGridView1.Rows[e.RowIndex].Cells["entry"];
			var entry = (AbsorberDatabaseEntry) (cell.Value);
			AbsorbentDoubleClickedEvent(entry);
		}

		private void dataGridView1_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyData)
			{
				case Keys.Back:
					textBoxSearch.Focus();
					SendKeys.SendWait("{BACKSPACE}");
					break;
				case (Keys.Control | Keys.Back):
					textBoxSearch.Focus();
					SendKeys.SendWait("^+{LEFT}{BACKSPACE}");
					break;
				case Keys.Enter:
					{
						if (dataGridView1.SelectedRows.Count < 1) return;

						object entry = dataGridView1.SelectedRows[0].Cells["entry"].Value;

						if (entry != null && entry.GetType() != typeof (DBNull))
							AbsorbentDoubleClickedEvent((AbsorberDatabaseEntry) entry);
						e.Handled = true;
					}
					break;
				case Keys.Up:
				case Keys.Down:
				case Keys.Left:
				case Keys.Right:
				case Keys.Home:
				case Keys.End:
				case Keys.ShiftKey | Keys.Shift:
					break;
				default:
					if (e.Shift || e.Control || e.Alt)
						break;
					_considerSend = true;

					break;
			}
		}

		private void dataGridView1_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (_considerSend)
			{
				textBoxSearch.Focus();
				SendKeys.SendWait(e.KeyChar.ToString(CultureInfo.InvariantCulture));
				e.Handled = true;
				_considerSend = false;
			}
		}

		/// <summary>
		/// Opens a dialog for the user to choose an absorption database file
		/// and assigns a method to take care of subsequent processing
		/// of the collected filename string
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ButtonLoadAbsorptionTextFileClick(object sender, EventArgs e)
		{
			_openAbsorptionTextFileDialog = new OpenFileDialog
			                                	{
			                                		DefaultExt = ".LI8",
			                                		CheckFileExists = true,
			                                		Multiselect = false,
			                                		Title = "Choose an absorption database text file"
			                                	};
			_openAbsorptionTextFileDialog.FileOk += OpenAbsorptionTextFileDialogFileOk;
			_openAbsorptionTextFileDialog.ShowDialog();
		}

		/// <summary>
		/// Loads default absorption database
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FormDatabaseMaterials_Load(object sender, EventArgs e) // TODO make more intelligent
		{
			LoadAbsorptionDatabaseAndPrepareDatagridview(
				"absorpsjonsdatabasen til multiconsult MATERIAL-MC-jan2008.LI8");
		}


		private void buttonClose_Click(object sender, EventArgs e)
		{
			Hide();
		}

		private void textBoxSearch_KeyDown(object sender, KeyEventArgs e)
		{
			if ((new[]
			     	{
			     		Keys.Up,
			     		Keys.Down,
			     		Keys.Enter,
			     	}).ToList().Contains(e.KeyData))
			{
				dataGridView1.Focus();
				if (e.KeyData == Keys.Up)
					SendKeys.SendWait("{UP}");
				if (e.KeyData == Keys.Down)
					SendKeys.SendWait("{DOWN}");
				e.Handled = true;
				e.SuppressKeyPress = true;
			}
		}

		#region Nested type: AbsorbentSelectedDelegate

		internal delegate void AbsorbentSelectedDelegate(AbsorberDatabaseEntry entry);

		#endregion
	}
}