﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using Cudafy;
using Cudafy.Host;
using Cudafy.Maths.FFT;
using ZedGraph;
using OlaCudafy1.FDTD;

namespace OlaCudafy1
{
	class OlaCufft
	{
		private ZedGraphControl _zed;
		private CSimulationSpace _simulationSpace;
		private GPGPU _gpu;

		public OlaCufft(ZedGraph.ZedGraphControl zg1)
		{
			_zed = zg1;

			if (_zed == null) throw new ArgumentNullException("Please supply a valid ZedGraph instance as argument.");

			//_zed.GraphPane.Title.Text = "Freqs";
			//_zed.GraphPane.XAxis.Title.Text = "Frequency (Hz)";
			//_zed.GraphPane.YAxis.Title.Text = "dB";
			//_zed.GraphPane.YAxis.Scale.Min = -60.0;
			//_zed.GraphPane.YAxis.Scale.Max = 0.0;
			//_zed.GraphPane.XAxis.Type = AxisType.Log;
			//_zed.GraphPane.YAxis.Type = AxisType.Log;
			//_zed.GraphPane.XAxis.Scale.Min = 20;
			
			//_zed.IsEnableVPan = false;
			//_zed.IsEnableHPan = false;
			//_zed.GraphPane.XAxis.Scale.IsPreventLabelOverlap = true;
			////_zed.GraphPane.XAxis.Scale.IsUseTenPower = false;
			////_zed.GraphPane.XAxis.Scale.MajorStep = Math.Log(2, 10);
			////_zed.GraphPane.XAxis.Scale.MinorStep = Math.Log(2, 10);
			////_zed.GraphPane.XAxis.Scale.Mag = 2;
		}

		internal void SetSimulationSpace(CSimulationSpace simulationSpace)
		{
			if (simulationSpace == null) 
				throw new ArgumentNullException
					("Please supply a valid instance for 'simulationSpace' argument (was null).");
			_simulationSpace = simulationSpace;

			_zed.GraphPane.XAxis.Scale.Max = _simulationSpace.GridSettings.GridMaxFreq;

			_simulationSpace.GenerationChanged += new GenerationChangedEventHandler(_simulationSpace_GenerationChanged);
		}

		void _simulationSpace_GenerationChanged(object sender, int generation)
		{
			UpdateGraph();
		}

		internal void SetGPU(GPGPU gpu)
		{
			if (gpu==null)
				throw new ArgumentNullException
					("Please supply a valid instance for 'gpu' argument (was null).");
			_gpu = gpu;
		}

		internal void UpdateGraph()
		{
			if (_simulationSpace == null) return;
			if (_gpu == null) return;

			//var gpgpufft =GPGPUFFT.Create(_gpu);
			//GPGPUFFT.Plan1D<float>();
			//FFTPlan1D plan1 = gpgpufft.Plan1D(eFFTType.Real2Complex, eDataType.Single, _simulationSpace.RecordingsLength);
			var t = new double[_simulationSpace.RecordingsLength];
			for (int i = 0; i < t.Length; i++)
			{
				t[i] = i*_simulationSpace.GridSettings.DT;
			}
			var s = new double[_simulationSpace.RecordingsLength];
			float[,] recordings = _simulationSpace.GetRecordings();
			const int micNumber = 0;
			for (int i = 0; i < s.Length; i++)
			{
				s[i] = recordings[micNumber, i];
			}
			var points = new BasicArrayPointList(t, s);

			_zed.GraphPane.CurveList.Clear();
			_zed.GraphPane.AddCurve("(label)", points, Color.CornflowerBlue);
			
			_zed.Invalidate();
		}

	}
}
