﻿using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

namespace OlaCudafy1
{
	/// <summary>
	/// Maps materials from model to absorbers in database
	/// Shall appear in a ListBox
	/// </summary>
	internal class WallAbsorberAssociations : IListSource, INotifyPropertyChanged
	{
		/// <summary>
		/// _absorberMapping[a] = b,
		/// means that material with dictionary number a
		/// is associated with absorber b
		/// i.e. absorber database entry pointed to by b
		/// </summary>
		private readonly IList<AbsorberDatabaseEntry> _absorberMappings;

		private readonly IList<string> _materialsArray;

		public WallAbsorberAssociations(IList<string> materialsArray)
		{
			_materialsArray = materialsArray;
			_absorberMappings = new AbsorberDatabaseEntry[Count];
		}


		public int Count
		{
			get { return _materialsArray.Count; }
		}

		public IList<string> MaterialsArray
		{
			get { return _materialsArray; }
		}

		/// <summary>
		/// Gets a list containing the entries from the absorber database which
		/// were chosen by the user. Here, a list item of index _i_
		/// is the absorber database entry whose absorption value or whatever
		/// should be used for material _i_
		/// </summary>
		public IList<AbsorberDatabaseEntry> AbsorberMappings
		{
			get { return _absorberMappings; }
		}


		//public string this[int index]
		//{
		//    get
		//    {
		//        string materialName = _materialsArray[index];
		//        if (_absorberMappings[index] == null)
		//            return materialName + ": ";
		//        return materialName + ": d=" + _absorberMappings[index].d;
		//    }
		//}

		#region Implementation of IListSource

		private IList _listBoxList;

		public IList GetList()
		{
			_listBoxList = new List<AssocListItem>(Count);
			for (int i = 0; i < Count; i++)
				_listBoxList.Add(new AssocListItem(this, i));
			return _listBoxList;
		}

		public bool ContainsListCollection
		{
			get { return false; }
		}

		#endregion

		#region Implementation of INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		#endregion

		internal void Associate(int indexToAssociate, AbsorberDatabaseEntry absorber)
		{
			_absorberMappings[indexToAssociate] = absorber;
			if (PropertyChanged != null)
				PropertyChanged(this, new PropertyChangedEventArgs("[]"));
		}
	}
}