﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace OlaCudafy1
{
	class WordWrapText
	{

		/// <summary>
		/// Wraps text to fit within 300 pixels if it were displayed with Calibri font at size 11 em.
		/// </summary>
		/// <param name="text"></param>
		/// <returns></returns>
		public static IEnumerable<string> WrapTextDefault(string text)
		{
			return WrapText(text, 110, "Calibri", 11);
		} 

		/// <summary>
		/// Wraps text to fit withing the given number of pixels horizontally, if it were to
		/// be displayed with the specified text font in the specified size in em.
		/// </summary>
		/// <param name="text">The text to be wrapped</param>
		/// <param name="pixels">Number of max pixels</param>
		/// <param name="fontFamily">Font family to use, for instance 'Calibri'</param>
		/// <param name="emSize"></param>
		/// <returns>The wrapped text as a list of lines</returns>
		public static IEnumerable<string> WrapText(string text, double pixels, string fontFamily, float emSize)
		{
			string[] words = text.Split(new string[] {" "}, StringSplitOptions.None);
			var wrappedLines = new List<string>();

			var actualLine = new StringBuilder();
			double actualWidth = 0;

			foreach (var item in words)
			{
				var formatted = new FormattedText(item, CultureInfo.CurrentCulture, System.Windows.FlowDirection.LeftToRight,
										  new Typeface(fontFamily), emSize, Brushes.Black);
				actualLine.Append(item + " ");
				actualWidth += formatted.Width;

				if (actualWidth <= pixels) continue;
				wrappedLines.Add(actualLine.Remove(actualLine.Length-1,1).ToString());
				actualLine.Clear();
				actualWidth = 0;
			}
			// add residuals aka last line
			if (actualLine.Length != 0)
				wrappedLines.Add(actualLine.Remove(actualLine.Length - 1, 1).ToString());

			return wrappedLines;
		}
	}
}
