struct MicrophoneStruct
{
	__device__ MicrophoneStruct()
	{
	}
	int X;
	int Y;
	int Z;
	__device__ MicrophoneStruct(int x, int y, int z)
	{
		X = x;
		Y = y;
		Z = z;
	}
};



struct Source
{
	__device__ Source()
	{
	}
	int X;
	int Y;
	int Z;
	float Frequency;
	float PressureAmplitude;
	int InvokeAtGeneration;
	int Type;
	__device__ Source(int x, int y, int z, float frequency, float pressureAmplitude, int invokeAtGeneration, int type)
	{
		X = x;
		Y = y;
		Z = z;
		Frequency = frequency;
		PressureAmplitude = pressureAmplitude;
		InvokeAtGeneration = invokeAtGeneration;
		Type = type;
	}
};



struct GridSettings
{
	__device__ GridSettings()
	{
	}
	float Boom;
	float CP;
	float CU;
	float DT;
	float DX;
	float FS;
	float GridMaxFreq;
	float GridNyquistFreq;
	int MinCellsPerWavelength;
	float MurCoefficient1;
	float MurCoefficient2;
	float MurCoefficient3;
	float _rmax;
	float R0;
	float Rk;
	float C0;
	float SoundPressure;
	float Impedance;
};



struct KindExtent
{
	__device__ KindExtent()
	{
	}
	int X;
	int Y;
	int Z;
};




// OlaCudafy1.FDTD.FDTDDrawMachine
__device__ int Clip(float *v);
// OlaCudafy1.FDTD.FDTDDrawMachine
__device__ int SetHSV(float h, float s, float v, unsigned char *outR, unsigned char *outG, unsigned char *outB);
// OlaCudafy1.FDTD.FDTDDrawMachine
extern "C" __global__ void FDTDDrawKernel(float* p, int pLen0, int pLen1, int pLen2, float* horiz, int horizLen0, int horizLen1, int horizLen2, float* vert, int vertLen0, int vertLen1, int vertLen2, float* zData, int zDataLen0, int zDataLen1, int zDataLen2, unsigned char* dest, int destLen0, int destLen1, int destLen2, unsigned char* mat, int matLen0, int matLen1, int matLen2, float scalePressure, float scaleBoth, Source* srcs, int srcsLen0, MicrophoneStruct* mics, int micsLen0, int displaySlice, int sliceDimension, KindExtent imgEx);

// OlaCudafy1.FDTD.FDTDDrawMachine
__device__ int Clip(float *v)
{
	if (v[0] < 0)
	{
		v[0] = 0;
	}
	else
	{
		if (v[0] > 1)
		{
			v[0] = 1;
		}
	}
	return 0;
}
// OlaCudafy1.FDTD.FDTDDrawMachine
__device__ int SetHSV(float h, float s, float v, unsigned char *outR, unsigned char *outG, unsigned char *outB)
{
	float num = 0;
	float num2 = 0;
	float num3 = 0;
	if (s == 0)
	{
		num3 = v;
		num2 = v;
		num = v;
	}
	else
	{
		if (h == 1)
		{
			h = 0;
		}
		float num4 = floorf(h * 6);
		int num5 = (int)num4;
		float num6 = h * 6 - num4;
		float num7 = v * (1 - s);
		float num8 = v * (1 - s * num6);
		float num9 = v * (1 - s * (1 - num6));
		switch (num5)
		{
			case 0:
			{
				num = v;
				num2 = num9;
				num3 = num7;
				break;
			}
			case 1:
			{
				num = num8;
				num2 = v;
				num3 = num7;
				break;
			}
			case 2:
			{
				num = num7;
				num2 = v;
				num3 = num9;
				break;
			}
			case 3:
			{
				num = num7;
				num2 = num8;
				num3 = v;
				break;
			}
			case 4:
			{
				num = num9;
				num2 = num7;
				num3 = v;
				break;
			}
			case 5:
			{
				num = v;
				num2 = num7;
				num3 = num8;
				break;
			}
		}
	}
	int num10 = (int)(256 * num);
	if (num10 > 255)
	{
		num10 = 255;
	}
	outR[0] = (unsigned char)num10;
	num10 = (int)(256 * num2);
	if (num10 > 255)
	{
		num10 = 255;
	}
	outG[0] = (unsigned char)num10;
	num10 = (int)(256 * num3);
	if (num10 > 255)
	{
		num10 = 255;
	}
	outB[0] = (unsigned char)num10;
	return 0;
}
// OlaCudafy1.FDTD.FDTDDrawMachine
extern "C" __global__ void FDTDDrawKernel(float* p, int pLen0, int pLen1, int pLen2, float* horiz, int horizLen0, int horizLen1, int horizLen2, float* vert, int vertLen0, int vertLen1, int vertLen2, float* zData, int zDataLen0, int zDataLen1, int zDataLen2, unsigned char* dest, int destLen0, int destLen1, int destLen2, unsigned char* mat, int matLen0, int matLen1, int matLen2, float scalePressure, float scaleBoth, Source* srcs, int srcsLen0, MicrophoneStruct* mics, int micsLen0, int displaySlice, int sliceDimension, KindExtent imgEx)
{
	int num = threadIdx.x + blockIdx.x * blockDim.x;
	int num2 = threadIdx.y + blockIdx.y * blockDim.y;
	if (num < imgEx.X && num2 < imgEx.Y)
	{
		int num3;
		int num4;
		int num5;
		switch (sliceDimension)
		{
			case 0:
			{
				num3 = displaySlice;
				num4 = num;
				num5 = num2;
				break;
			}
			case 1:
			{
				num3 = num2;
				num4 = displaySlice;
				num5 = num;
				break;
			}
			default:
			{
				num3 = num;
				num4 = num2;
				num5 = displaySlice;
				break;
			}
		}
		if (num3 >= 0 && num4 >= 0)
		{
			unsigned char b = (0 < num) ? 1 : 0;
			float num6;
			switch (sliceDimension)
			{
				case 0:
				{
					num6 = (vert[(num3) * vertLen1 * vertLen2 + ( num4) * vertLen2 + ( num5)] + vert[(num3) * vertLen1 * vertLen2 + ( num4 - (int)b) * vertLen2 + ( num5)]) / 2;
					break;
				}
				case 1:
				{
					num6 = (zData[(num3) * zDataLen1 * zDataLen2 + ( num4) * zDataLen2 + ( num5)] + zData[(num3) * zDataLen1 * zDataLen2 + ( num4) * zDataLen2 + ( num5 - (int)b)]) / 2;
					break;
				}
				default:
				{
					num6 = (horiz[(num3) * horizLen1 * horizLen2 + ( num4) * horizLen2 + ( num5)] + horiz[(num3 - (int)b) * horizLen1 * horizLen2 + ( num4) * horizLen2 + ( num5)]) / 2;
					break;
				}
			}
			unsigned char b2 = (0 < num2) ? 1 : 0;
			float num7;
			switch (sliceDimension)
			{
				case 0:
				{
					num7 = (zData[(num3) * zDataLen1 * zDataLen2 + ( num4) * zDataLen2 + ( num5)] + zData[(num3) * zDataLen1 * zDataLen2 + ( num4) * zDataLen2 + ( num5 - (int)b2)]) / 2;
					break;
				}
				case 1:
				{
					num7 = (horiz[(num3) * horizLen1 * horizLen2 + ( num4) * horizLen2 + ( num5)] + horiz[(num3 - (int)b2) * horizLen1 * horizLen2 + ( num4) * horizLen2 + ( num5)]) / 2;
					break;
				}
				default:
				{
					num7 = (vert[(num3) * vertLen1 * vertLen2 + ( num4) * vertLen2 + ( num5)] + vert[(num3) * vertLen1 * vertLen2 + ( num4 - (int)b2) * vertLen2 + ( num5)]) / 2;
					break;
				}
			}
			float num8 = 413.006317 * sqrtf(num6 * num6 + num7 * num7);
			float num9 = p[(num3) * pLen1 * pLen2 + ( num4) * pLen2 + ( num5)] * scalePressure;
			num9 *= scaleBoth;
			num8 *= scaleBoth;
			float num10 = 0;
			float num11 = 0;
			float num12 = 0;
			num11 = num8;
			num10 = num9 * 0.5 + 0.5;
			Clip(&num11);
			num11 = num11 * 0.6 + 0.4;
			Clip(&num10);
			num10 = num10 * 0.7 + 0.2;
			num12 += 0.1 * num10;
			Clip(&num12);
			num12 *= 0.6666667;
			if (mat[(num3) * matLen1 * matLen2 + ( num4) * matLen2 + ( num5)] != 0)
			{
				num10 = 0;
				num11 = 1;
			}
			unsigned char b3;
			unsigned char b4;
			unsigned char b5;
			SetHSV(num12, num10, num11, &b3, &b4, &b5);
			dest[(num) * destLen1 * destLen2 + ( num2) * destLen2 + (0)] = b5;
			dest[(num) * destLen1 * destLen2 + ( num2) * destLen2 + (1)] = b4;
			dest[(num) * destLen1 * destLen2 + ( num2) * destLen2 + (2)] = b3;
			dest[(num) * destLen1 * destLen2 + ( num2) * destLen2 + (3)] = 255;
		}
	}
}
